# Copyright (c) 2020-2021 Nokia Corporation and/or its subsidiary(-ies). All rights reserved.
# This file is covered by the license agreement found in the file “license.txt” in the root of this project.

#
# this script calcuates average psnr and msssim scores for input images and reconstructed images


import pandas as pd
import glob
import os
import cv2
import numpy as np
import torch
import pytorch_msssim 
from multiprocessing import Pool
import itertools

import utils 

# read config file
cfg = utils.read_config()

QP_list = cfg.qps.split()
output_dir = cfg.output_dir

# image directory
img_dir = '../dataset/validation/validation'

# annotations
dataset_dir='../dataset'
annot_dir = os.path.join(dataset_dir, "annotations_5k")

input_img_list = []
if 'detection' in cfg.task.split(): 
  input_img_list.append(os.path.join(annot_dir, 'detection_validation_input_5k.lst'))
if 'segmentation' in cfg.task.split():
  input_img_list.append(os.path.join(annot_dir, 'segmentation_validation_input_5k.lst'))


output_data_file = os.path.join(output_dir, 'psnr_msssim_data.csv')
output_report_file  = os.path.join(output_dir, 'psnr_msssim_report.csv')


###############################################

# get validation image ids
val_img_ids = set()
for img_list_fname in input_img_list:
  with open(img_list_fname) as f:
    imgs = f.read().splitlines()
    ids = [os.path.splitext(x)[0] for x in imgs]
    val_img_ids = val_img_ids.union(ids)

#for img_id in sorted(val_img_ids):
def process_img(img_id):
  img_psnr_msssim_data = []
  print(f'processing {img_id} ...')
  img_fname = os.path.join(img_dir, img_id+'.jpg')
  img = cv2.imread(img_fname).astype(float) #HWC
  img_batch = torch.tensor(img).unsqueeze(0).permute(0,3,1,2).cuda()

  for QP in QP_list:
    recon_img_fname = os.path.join(cfg.output_dir,'rescaled',QP,img_id+'.png')
    recon_img = cv2.imread(recon_img_fname).astype(float) #HWC

    # calculate PSNR
    mse = np.mean((img - recon_img)**2)
    if mse == 0: 
      psnr = 100
    else:
      psnr = 20 * np.log10(255 / np.sqrt(mse))

    # calculate ssim and msssim
    recon_img_batch = torch.tensor(recon_img).unsqueeze(0).permute(0,3,1,2).cuda()
    ssim = pytorch_msssim.ssim(img_batch, recon_img_batch, data_range=255).item()
    msssim = pytorch_msssim.ms_ssim(img_batch, recon_img_batch, data_range=255).item()
    
    img_psnr_msssim_data.append({ \
      'ImageID': img_id,
      'QP': QP,
      'PSNR': psnr,
      'SSIM': ssim,
      'MSSSIM': msssim})
  return img_psnr_msssim_data

#for img_id in sorted(val_img_ids):
with Pool(5) as p:
  psnr_msssim_data = p.map(process_img, list(val_img_ids))

psnr_msssim_data = list(itertools.chain(*psnr_msssim_data))

psnr_msssim_data = pd.DataFrame(psnr_msssim_data)
psnr_msssim_data.to_csv(output_data_file, index=False)
 

# 
psnr_msssim_data = pd.read_csv(output_data_file)
report = []
#detection
for img_list in input_img_list:
  print('processing', img_list)
  with open(img_list, 'r') as f:
    img_id_list=[x[:-4] for x in f.read().splitlines()] #remove .png
  
  for QP in QP_list:
    sub_data = psnr_msssim_data.loc[psnr_msssim_data['ImageID'].isin(img_id_list) & \
      (psnr_msssim_data['QP']==QP)]

    report.append({ \
      'ImageList': os.path.basename(img_list),
      'QP': QP,
      'PSNR': sub_data['PSNR'].mean(),
      'SSIM': sub_data['SSIM'].mean(),
      'MSSSIM': sub_data['MSSSIM'].mean()})

report = pd.DataFrame(report)
report.to_csv(output_report_file, index=False)

print(report)


